// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/shader/ShaderProgram.hpp"
#include "scene/components/display/MeshRenderer.hpp"

namespace Dawn {
  class Material;

  struct ShaderPass {
    ShaderProgram *shaderProgram = nullptr;
    int32_t orderShader = 0;
    bool_t needsW = false;
    
    // Parameters
    std::map<shaderparameter_t, struct Color> colorValues;
    std::map<shaderparameter_t, bool_t> boolValues;
    std::map<shaderparameter_t, glm::mat4> matrixValues;
    std::map<shaderparameter_t, glm::vec3> vec3Values;
    std::map<shaderparameter_t, textureslot_t> textureValues;
    std::map<shaderparameter_t, float_t> floatValues;

    // Textures
    std::map<textureslot_t, Texture*> textureSlots;
  };

  class Shader {
    public:
      int_fast16_t renderId = 0;

      /**
       * Compile all programs for this shader. This amy not remain forever as I
       * may decide to allow shaders to share programs somehow. For now though
       * this is clean enough.
       */
      virtual void compile() = 0;

      /**
       * Returns the list of passes to render for the given scene item.
       * 
       * @param mesh Mesh Renderer for the scene item.
       * @param material Material for the scene item.
       * @return List of passes to render.
       */
      virtual std::vector<struct ShaderPass> getItemPasses(
        MeshRenderer *mesh,
        Material *material
      ) = 0;

      /**
       * Called once per frame, set the global shader parameters that is used by
       * every item in the scene.
       * 
       * @param cameraProjection Projection matrix of the camera.
       * @param cameraView View matrix of the camera.
       */
      virtual void setGlobalParameters(
        glm::mat4 cameraProjection,
        glm::mat4 cameraView
      ) = 0;
  };
}